/*
 * Copyright:
 * ----------------------------------------------------------------------------
 * This confidential and proprietary software may be used only as authorized
 * by a licensing agreement from ARM Limited.
 *      (C) COPYRIGHT 2018 ARM Limited, ALL RIGHTS RESERVED
 * The entire notice above must be reproduced on all authorized copies and
 * copies may only be made to the extent permitted by a licensing agreement
 * from ARM Limited.
 * ----------------------------------------------------------------------------
 */

package core

import (
	"path/filepath"

	"github.com/google/blueprint"
)

// Support generation of static and shared libraries
// This file declares the common properties and functions needed by both.

type GenerateLibraryProps struct {
	// List of headers that are created (if any)
	Headers []string
}

type GenerateLibrary struct {
	GenerateCommon
	Properties struct {
		GenerateLibraryProps
	}
}

type GenerateLibraryInterface interface {
	blueprint.Module
	DependentInterface

	libExtension() string
	getSources(ctx blueprint.ModuleContext) []string
}

//// Local functions

// Return a list of shared libraries generated by this module with full paths to the generated location
func getLibraryGeneratedPath(m GenerateLibraryInterface, g GeneratorBackend) string {
	return filepath.Join(m.OutputDir(g), m.Name()+m.libExtension())
}

// Map sources to outputs. This is primarily to support
// TransformSource, so here we return a single element associating all
// inputs with all outputs
func inouts(m GenerateLibraryInterface, ctx blueprint.ModuleContext) []Inout {
	var inout Inout
	g := getBackend(ctx)
	inout.src_in = m.getSources(ctx)
	inout.gen_in = getGeneratedFiles(ctx)
	inout.out = m.Outputs(g)
	return []Inout{inout}
}

//// Support Splittable

func (m *GenerateLibrary) supportedVariants() []string {
	return []string{m.GenerateCommon.Properties.Target}
}

func (m *GenerateLibrary) disable() {
	// This should never actually be called, as we will always support one target
	panic("disable() called on GenerateLibrary")
}

func (m *GenerateLibrary) setVariant(variant string) {
	// No need to actually track this, as a single target is always supported
}

func (m *GenerateLibrary) getSplittableProps() *SplittableProps {
	return &m.GenerateCommon.Properties.FlagArgsBuild.SplittableProps
}
