/*
 * Copyright:
 * ----------------------------------------------------------------------------
 * This confidential and proprietary software may be used only as authorized
 * by a licensing agreement from ARM Limited.
 *      (C) COPYRIGHT 2018 ARM Limited, ALL RIGHTS RESERVED
 * The entire notice above must be reproduced on all authorized copies and
 * copies may only be made to the extent permitted by a licensing agreement
 * from ARM Limited.
 * ----------------------------------------------------------------------------
 */

package core

import (
	"reflect"

	"github.com/google/blueprint"

	"bob/utils"
)

type GenerateSharedLibrary struct {
	GenerateLibrary
}

//// Support GenerateLibraryInterface

func (m *GenerateSharedLibrary) libExtension() string {
	return ".so"
}

//// Support PhonyInterface, DependentInterface

// List of everything generated by this target
func (m *GenerateSharedLibrary) Outputs(g GeneratorBackend) []string {
	out := []string{getLibraryGeneratedPath(m, g)}
	out = append(out, utils.PrefixDirs(m.Properties.Headers, m.OutputDir(g))...)
	return out
}

//// Support Installable

func (m *GenerateSharedLibrary) FilesToInstall(ctx blueprint.ModuleContext) []string {
	return []string{getLibraryGeneratedPath(m, getBackend(ctx))}
}

//// Support blueprint.Module

func (m *GenerateSharedLibrary) GenerateBuildActions(ctx blueprint.ModuleContext) {
	if IsEnabled(m) {
		inouts := inouts(m, ctx)
		getBackend(ctx).GenSharedActions(m, ctx, inouts)
	}
}

//// Factory functions

func genSharedLibFactory(config *Config) (blueprint.Module, []interface{}) {
	module := &GenerateSharedLibrary{}
	module.GenerateCommon.Properties.Features.Init(config, reflect.TypeOf(GenerateProps{}))
	return module, []interface{}{
		&module.SimpleName.Properties,
		&module.GenerateCommon.Properties,
		&module.Properties,
	}
}
