/*
 * Copyright:
 * ----------------------------------------------------------------------------
 * This confidential and proprietary software may be used only as authorized
 * by a licensing agreement from ARM Limited.
 *      (C) COPYRIGHT 2017-2018 ARM Limited, ALL RIGHTS RESERVED
 * The entire notice above must be reproduced on all authorized copies and
 * copies may only be made to the extent permitted by a licensing agreement
 * from ARM Limited.
 * ----------------------------------------------------------------------------
 */

package core

import (
	"errors"
	"fmt"
)

func GetAssembler(p ConfigProvider, target_type string) (compiler string, compilerflags string) {
	props := getConfig(p).Properties
	root := props.GetString("as_binary")

	/* TOOLCHAIN_CLANG will use the GNU assembler */
	if target_type != HOST_TAG {
		compiler = props.GetString("toolchain_prefix") + root
	} else {
		compiler = root
	}

	return
}

func GetCCompiler(p ConfigProvider, target_type string) (compiler string, compilerflags string) {
	props := getConfig(p).Properties
	if props.GetBool("toolchain_clang") {
		compiler = props.GetString("clang_binary")
		if target_type != HOST_TAG {
			if props.GetString("clang_target") != "" {
				compilerflags += fmt.Sprintf(" -target %s",
					props.GetString("clang_target")) //TODO - support targets
			}
			if props.GetString("clang_sysroot") != "" {
				compilerflags += fmt.Sprintf(" --sysroot %s",
					props.GetString("clang_sysroot"))
			}
		}
	} else if props.GetBool("toolchain_gnu") {
		if target_type != HOST_TAG {
			compiler = fmt.Sprintf("%s%s", props.GetString("toolchain_prefix"),
				props.GetString("gcc_binary"))
			compilerflags = props.GetString("gcc_target_flags")
		} else {
			compiler = props.GetString("gcc_binary")
		}
	} else {
		panic(errors.New("no usable compiler toolchain configured"))
	}

	return
}

func GetCXXCompiler(p ConfigProvider, target_type string) (compiler string, compilerflags string) {
	props := getConfig(p).Properties
	if props.GetBool("toolchain_clang") {
		compiler = props.GetString("clangxx_binary")

		if target_type != HOST_TAG {
			if props.GetString("clang_target") != "" {
				compilerflags += fmt.Sprintf(" -target %s",
					props.GetString("clang_target"))
			}
			if props.GetString("clang_sysroot") != "" {
				if props.GetString("clang_target") == "" {
					panic(errors.New("CLANG_TARGET is not set"))
				}
				if props.GetString("target_toolchain_version") == "" {
					panic(errors.New("TARGET_TOOLCHAIN_VERSION is not set"))
				}
				compilerflags += fmt.Sprintf(" --sysroot %s",
					props.GetString("clang_sysroot"))
				compilerflags += fmt.Sprintf(" -isystem %s/../include/c++/%s ",
					props.GetString("clang_sysroot"),
					props.GetString("target_toolchain_version"))
				compilerflags += fmt.Sprintf(" -isystem %s/../include/c++/%s/%s ",
					props.GetString("clang_sysroot"),
					props.GetString("target_toolchain_version"),
					props.GetString("clang_target"))
			}
		}
	} else if props.GetBool("toolchain_gnu") {
		if target_type != HOST_TAG {
			compiler = fmt.Sprintf("%s%s", props.GetString("toolchain_prefix"),
				props.GetString("gxx_binary"))
			compilerflags = props.GetString("gcc_target_flags")
		} else {
			compiler = props.GetString("gxx_binary")
		}
	} else {
		panic(errors.New("no usable compiler toolchain configured"))
	}
	return
}

func GetArchiver(p ConfigProvider, target_type string) (ar string) {
	props := getConfig(p).Properties
	root := props.GetString("ar_binary")

	/* TOOLCHAIN_CLANG will use the GNU archiver */
	if target_type != HOST_TAG {
		ar = props.GetString("toolchain_prefix") + root
	} else {
		ar = root
	}

	return
}
