# Copyright:
# ----------------------------------------------------------------------------
# This confidential and proprietary software may be used only as authorized
# by a licensing agreement from ARM Limited.
#      (C) COPYRIGHT 2018 ARM Limited, ALL RIGHTS RESERVED
# The entire notice above must be reproduced on all authorized copies and
# copies may only be made to the extent permitted by a licensing agreement
# from ARM Limited.
# ----------------------------------------------------------------------------
#

import logging
import os

logger = logging.getLogger(__name__)

g_kernel_configs = dict()


def option_enabled(kdir, option):
    """Return true if a given kernel config option is enabled"""
    global g_kernel_configs

    if kdir not in g_kernel_configs:
        config_file = os.path.join(kdir, '.config')
        config = dict()
        try:
            with open(config_file, "rt") as fp:
                for line in fp.readlines():
                    try:
                        (key, val) = line.split("=")
                        config[key.strip()] = val.strip()
                    except ValueError:
                        pass
        except IOError as e:
            logger.warning("Failed to open the kernel config file in %s: Couldn't check for option %s",
                            config_file, option)
        g_kernel_configs[kdir] = config

    return g_kernel_configs[kdir].get(option) == 'y'


def get_arch(kdir):
    arch_dir = os.path.join(kdir, "arch")
    # Each directory in $KDIR/arch has a config option with the same name.
    for arch in os.listdir(arch_dir):
        if not os.path.isfile(os.path.join(arch_dir, arch, "Kconfig")):
            continue

        if option_enabled(kdir, "CONFIG_" + arch.upper()):
            return arch

    if option_enabled(kdir, "CONFIG_X86_32"):
        return "i386"
    elif option_enabled(kdir, "CONFIG_X86_64"):
        return "x86_64"
    elif option_enabled(kdir, "CONFIG_PPC32") or option_enabled(kdir, "CONFIG_PPC64"):
        return "powerpc"
    elif (option_enabled(kdir, "CONFIG_SUPERH") or option_enabled(kdir, "CONFIG_SUPERH32") or
          option_enabled(kdir, "CONFIG_SUPERH64")):
        return "sh"

    logger.warning("Couldn't get ARCH for kernel %s", kdir)
    return None
