#!/bin/bash
#
# Copyright:
# ----------------------------------------------------------------------------
# This confidential and proprietary software may be used only as authorized
# by a licensing agreement from ARM Limited.
#      (C) COPYRIGHT 2016-2018 ARM Limited, ALL RIGHTS RESERVED
# The entire notice above must be reproduced on all authorized copies and
# copies may only be made to the extent permitted by a licensing agreement
# from ARM Limited.
# ----------------------------------------------------------------------------

# Some subtleties:
#
# * Calling bootstrap_linux with an absolute path means SRCDIR is
#   absolute, and you can thus move BUILDDIR
#   independently of SRCDIR.
#   - this doesn't work
#
# * Calling bootstrap_linux with a relative path means SRCDIR is
#   relative, and you can move SRCDIR and BUILDDIR together.
#
# Both of the above depend on the ninja rules being written correctly.

set -e

SCRIPT_DIR="$(dirname "${BASH_SOURCE[0]}")"

# Due to issues in Bob SRCDIR must be absolute. Because it's derived from
# SCRIPT_DIR, make SCRIPT_DIR absolute so that we get SRCDIR for free.
if [ "${SCRIPT_DIR:0:1}" != '/' ]; then
    SCRIPT_DIR="$(readlink -f "${SCRIPT_DIR}")"
fi

# Select a BUILDDIR if not provided one
if [[ -z "$BUILDDIR" ]]; then
    echo "BUILDDIR is not set - using build"
    BUILDDIR=build
fi

# Create the build directory. The 'relative_path' helper can only be used on
# existing directories.
mkdir -p "$BUILDDIR"

source "${SCRIPT_DIR}/bob/pathtools.bash"

ORIG_BUILDDIR="${BUILDDIR}"

# Move to the source directory - we want this to be the working directory of the build
SRCDIR="$(dirname "${SCRIPT_DIR}")"
if [ "${BUILDDIR:0:1}" != '/' ]; then
    # Redo BUILDDIR to be relative to SRCDIR
    BUILDDIR="$(relative_path "${SRCDIR}" "${BUILDDIR}")"
fi
cd "${SRCDIR}"

# Set variables to bootstrap Bob.
export TOPNAME="build.bp"
export BLUEPRINT_LIST_FILE="${BUILDDIR}/.bplist"
export CONFIGNAME="mali.config"
export BOB_CONFIG_OPTS="--ignore-missing"
export BOB_CONFIG_PLUGINS="bldsys/tools/kernel_explore bldsys/tools/tools_checker_plugin bldsys/tools/ddk_host_explore"
export SRCDIR
export BUILDDIR

bldsys/bob/bootstrap_linux.bash

# Pick up some info that bob has worked out
BOOTSTRAP=".bob.bootstrap"
source "${BUILDDIR}/${BOOTSTRAP}"

# Setup the DDK build script
if [ "${SRCDIR:0:1}" != '/' ]; then
    # Use a relative symlink
    if [ "${SRCDIR}" != '.' ]; then
        ln -sf "${WORKDIR}/${SRCDIR}/bldsys/buildme.bash" "${BUILDDIR}/buildme"
    else
        ln -sf "${WORKDIR}/bldsys/buildme.bash" "${BUILDDIR}/buildme"
    fi
else
    # Use an absolute symlink
    ln -sf "${SRCDIR}/bldsys/buildme.bash" "${BUILDDIR}/buildme"
fi

# Print info for users
echo "To configure the build directory, run ${ORIG_BUILDDIR}/config ARGS"
echo "Then build with ${ORIG_BUILDDIR}/buildme"
