# This confidential and proprietary software may be used only as
# authorised by a licensing agreement from ARM Limited
# (C) COPYRIGHT 2017-2018 ARM Limited
# ALL RIGHTS RESERVED
# The entire notice above must be reproduced on all authorised
# copies and copies may only be made to the extent permitted
# by a licensing agreement from ARM Limited.

# Define GPU version info centrally
# Note: strictly midgard doesn't define product. Values defined here purely to order the midgard products
gpu_info = {
    "t60x" : { "arch" : "midgard",
               "arch_major" : 5,
               "arch_minor" : 5,
               "arch_product" : 0
           },
    "t62x" : { "arch" : "midgard",
               "arch_major" : 5,
               "arch_minor" : 5,
               "arch_product" : 1
           },
    "t72x" : { "arch" : "midgard",
               "arch_major" : 5,
               "arch_minor" : 5,
               "arch_product" : 2
           },
    "t76x" : { "arch" : "midgard",
               "arch_major" : 5,
               "arch_minor" : 5,
               "arch_product" : 3
           },
    "t82x" : { "arch" : "midgard",
               "arch_major" : 5,
               "arch_minor" : 5,
               "arch_product" : 4
           },
    "t83x" : { "arch" : "midgard",
               "arch_major" : 5,
               "arch_minor" : 5,
               "arch_product" : 5
           },
    "t86x" : { "arch" : "midgard",
               "arch_major" : 5,
               "arch_minor" : 5,
               "arch_product" : 6
           },
    "tFRx" : { "arch" : "midgard",
               "arch_major" : 5,
               "arch_minor" : 5,
               "arch_product" : 7
           },

    "tMIx": { "arch" : "bifrost",
              "arch_major" : 6,
              "arch_minor" : 0,
              "arch_product" : 0
          },
    "tHEx": { "arch" : "bifrost",
              "arch_major" : 6,
              "arch_minor" : 2,
              "arch_product" : 1
          },
    "tSIx": { "arch" : "bifrost",
              "arch_major" : 7,
              "arch_minor" : 0,
              "arch_product" : 0
          },
    "tDVx": { "arch" : "bifrost",
              "arch_major" : 7,
              "arch_minor" : 0,
              "arch_product" : 3
          },
    "tNOx": { "arch" : "bifrost",
              "arch_major" : 7,
              "arch_minor" : 2,
              "arch_product" : 1
          },
    "tGOx_r0p0": {
               "arch" : "bifrost",
               "arch_major" : 7,
               "arch_minor" : 2,
               "arch_product" : 2,
               "canonical_name" : "tGOx"
          },
    "tGOx_r1p0": {
               "arch" : "bifrost",
               "arch_major" : 7,
               "arch_minor" : 4,
               "arch_product" : 2,
               "canonical_name" : "tGOx"
          },
    "tKAx": { "arch" : "bifrost",
              "arch_major" : 8,
              "arch_minor" : 0,
              "arch_product" : 0
          },
    "tEGx": { "arch" : "bifrost",
              "arch_major" : 7,
              "arch_minor" : 2,
              "arch_product" : 1
          },
    "tTRx": { "arch" : "bifrost",
              "arch_major" : 9,
              "arch_minor" : 0,
              "arch_product" : 6
          },
    "tBOx": { "arch" : "bifrost",
              "arch_major" : 8,
              "arch_minor" : 0,
              "arch_product" : 2
          }
}

gpu_aliases = {
    "t88x" : "tFRx",
    "g71"  : "tMIx",
    "g51"  : "tSIx",
    "g72"  : "tHEx",
    "g52_r0p0" : "tGOx_r0p0",
    "g52_r1p0" : "tGOx_r1p0",
    "g31" : "tDVx",
    "g76" : "tNOx",
}

# Lookup table of the supported features.
# This is an initial table defining high level features.
# The table can be updated by components if needed.
gpu_features = {
    't60x': [                                ], # t60x
    't62x': [                                ], # t62x
    't72x': [                                ], # t72x
    't76x': [ 'mfbd', 'afbc'                 ], # t76x
    't82x': [ 'mfbd', 'afbc'                 ], # t82x
    't83x': [ 'mfbd', 'afbc'                 ], # t83x
    't86x': [ 'mfbd', 'afbc'                 ], # t86x
    'tFRx': [ 'mfbd', 'afbc'                 ], # t88x/tFRx
    'tMIx': [ 'mfbd', 'afbc', 'cl20'         ], # g71/tMIx
    'tHEx': [ 'mfbd', 'afbc', 'cl20'         ], # g72/tHEx
    'tSIx': [ 'mfbd', 'afbc', 'cl20'         ], # g51/tSIx
    'tDVx': [ 'mfbd', 'afbc', 'cl20'         ], # tDVx
    'tNOx': [ 'mfbd', 'afbc', 'cl20', 'cl21' ], # tNOx
    'tGOx_r0p0': [ 'mfbd', 'afbc', 'cl20', 'cl21' ], # tGOx r0p0
    'tGOx_r1p0': [ 'mfbd', 'afbc', 'cl20', 'cl21' ], # tGOx r1p0
    'tKAx': [ 'mfbd', 'afbc', 'cl20', 'cl21' ], # tKAx
    'tTRx': [ 'mfbd', 'afbc', 'cl20', 'cl21' ], # tTRx
    'tEGx': [ 'mfbd', 'afbc', 'cl20', 'cl21' ], # tEGx
    'tBOx': [ 'mfbd', 'afbc', 'cl20', 'cl21' ]  # tBOx
}

# List of known features used to prevent modification
gpu_feature_list = set([feature for sublist in gpu_features.itervalues() for feature in sublist])

# Return a list of known GPUs
def get_list():
    gpu_list = gpu_info.keys()
    gpu_list.extend(gpu_aliases.keys())
    return gpu_list

# Does string refer to a known GPU
def is_known(string):
    return string in get_list()

# Retrieve the GPU architecture name as a string
def get_arch(gpu):
    return gpu_info[gpu]["arch"]

# Is gpu in the Midgard family of products
def is_midgard(gpu):
    return gpu_info[gpu]["arch"] == "midgard"

# Is gpu in the Bifrost family of products
def is_bifrost(gpu):
    return gpu_info[gpu]["arch"] == "bifrost"

# Retrieve the GPU name used within the codebase as a string
def get_internal_name(gpu):
    return gpu_aliases.get(gpu, gpu)

# Retrieve the GPU name used within the codebase as a string, disregarding
# the revision numbers
def get_canonical_name(gpu):
    if "canonical_name" in gpu_info[gpu]:
        return gpu_info[gpu]["canonical_name"]

    return gpu

# Retrieve the major and minor architecture version of the GPU
def get_arch_major_minor(gpu):
    return (gpu_info[gpu]["arch_major"], gpu_info[gpu]["arch_minor"])

# Return an integer that uniquely identifies the product, and can be used in comparisons.
# The format of this integer is internal, and not expected to be used anywhere else.
def get_version(gpu):
    return ((gpu_info[gpu]["arch_major"] << 8) |
            (gpu_info[gpu]["arch_product"] << 4) | gpu_info[gpu]["arch_minor"])

# Compare GPUs
# returns -ve if gpu1 is earlier than gpu2
# returns +ve if gpu1 is later than gpu2
# returns 0 if gpu1 and gpu2 are the same
def compare(gpu1, gpu2):
    ver1 = get_version(gpu1)
    ver2 = get_version(gpu2)
    return ver1 - ver2

# Return an integer that identifies the architecture used by the GPU.
# The format of this integer is internal, and not expected to be used anywhere else.
def get_arch_version(gpu):
    return (gpu_info[gpu]["arch_major"] << 8 | gpu_info[gpu]["arch_minor"])

# Compare GPUs by architecture version only (ignoring product ID)
# returns -ve if gpu1 is earlier than gpu2
# returns +ve if gpu1 is later than gpu2
# returns 0 if gpu1 and gpu2 are the same
def compare_arch_version(gpu1, gpu2):
    ver1 = get_arch_version(gpu1)
    ver2 = get_arch_version(gpu2)
    return ver1 - ver2

# Checks if the feature supported on the currently selected GPU
def is_feature_supported(gpu, feature):
    return feature in gpu_features[gpu];

# Add a feature and update specific gpus
# This allows components to update the main feature list to be referenced elsewhere
def add_feature(feature, gpu_list):
    if feature in gpu_feature_list:
        print("Warning: ignoring redefinition of GPU feature {0}".format(feature))
    else:
        gpu_feature_list.add(feature)

        for gpu in gpu_list:
            gpu_features[gpu].append(feature)

# Add a feature and update specific gpus within a specific range (inclusive)
# This allows components to update the main feature list to be referenced elsewhere
def add_feature_range(feature, first_gpu, last_gpu=None):
    if feature in gpu_feature_list:
        print("Warning: ignoring redefinition of GPU feature {0}".format(feature))
    else:
        gpu_feature_list.add(feature)

        for gpu, features in gpu_features.iteritems():
            if compare(gpu, first_gpu) >= 0 and (last_gpu is None or compare(gpu, last_gpu) <= 0):
                features.append(feature)
