# Copyright:
# ----------------------------------------------------------------------------
# This confidential and proprietary software may be used only as authorized
# by a licensing agreement from ARM Limited.
#      (C) COPYRIGHT 2018 ARM Limited, ALL RIGHTS RESERVED
# The entire notice above must be reproduced on all authorized copies and
# copies may only be made to the extent permitted by a licensing agreement
# from ARM Limited.
# ----------------------------------------------------------------------------
#

import os
import sys
import re
import subprocess
import logging
from config_system import get_config, set_config
sys.path.append(os.path.join(os.path.dirname(os.path.realpath(sys.argv[0])), "../bob/scripts"))
import kernel_config_parser

logger = logging.getLogger(__name__)

g_kernel_version = None
def kernel_get_version(kdir):
    global g_kernel_version

    if g_kernel_version:
        return g_kernel_version

    cmd = ['make', '-s', 'kernelversion']
    try:
        version = subprocess.check_output(cmd, cwd = kdir).strip()
        version = version.decode(sys.getdefaultencoding())
    except (OSError, subprocess.CalledProcessError) as e:
        logger.warning("Couldn't get kernel version: %s" % str(e))
        version = "0.0.0"

    m = re.match('([0-9]+)(?:\.([0-9]+)(?:\.([0-9]+))?)?(.*)', version)
    if not m:
        logger.warning("Couldn't parse kernel version '%s'" % version)

    version_ints = []
    for i in m.groups()[0:3]:
        if i != None and i.isdigit():
            version_ints.append(int(i))
        else: # Not set
            version_ints.append(0)

    g_kernel_version = tuple(version_ints)
    return g_kernel_version

def disable_devfreq_if_not_supported(kdir):
    if get_config('MALI_DEVFREQ')['value'] != 'y':
        return

    if kernel_get_version(kdir) >= (3, 18, 0):
        if kernel_config_parser.option_enabled(kdir, 'CONFIG_PM_DEVFREQ'):
            # Leave MALI_DEVFREQ at its current value
            return
        else:
            logger.warning("Disabling MALI_DEVFREQ - CONFIG_PM_DEVFREQ not enabled in kernel")
    else:
        logger.warning("Disabling MALI_DEVFREQ - kernel is too old")
    set_config('MALI_DEVFREQ', 'n')

def plugin_exec():
    if get_config('ALLOW_KERNEL_EXPLORE')['value'] != 'y':
        return

    kdir = get_config('KERNEL_DIR')['value']
    if get_config('BUILD_KERNEL_MODULES')['value'] == 'y' or get_config('USERSPACE_INCLUDES_KERNEL_HEADERS')['value'] == 'y':
        if kdir == "":
            logger.warning("KERNEL_DIR not set, kernel exploration disabled")
            return
    if get_config('BUILD_KERNEL_MODULES')['value'] == 'y':
        disable_devfreq_if_not_supported(kdir)


