#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/device.h>
#include <linux/fs.h>
#include <linux/delay.h>
#include <mach/io.h>          /* rtd_xx */
#include <linux/version.h>
#include <linux/fs.h>
#include <linux/proc_fs.h>
#include <linux/kobject.h> 	//sys ctl
#include <mali_kbase_pm.h>

#define SYS_REG_BASE	       (0xB8000000)
#define REG_GPU_BASE           (0xB8100000)

#define SYS_REG_GPU_DVFS_OFFSET         0x0448
#define SYS_REG_GPU_DVFS_OE_BIT         0x01
#define SYS_REG_GPU_DVFS_OE_DONE_BIT    0x02
#define SYS_REG_GPU_PLL_GPU2_OFFSET     0x0454
#define SYS_REG_GPU_PLL_GPU1_OFFSET   0x0450  
static DEFINE_MUTEX(gGPU_DFS_LOCK);

struct kbase_device *gkbdev=NULL;
		
/*
0: disable( ignore thermal check, only support manual change )
1: auto (default) (thermal warning will overwrite current manual clock setting)
*/
static unsigned int gGPU_DFS_MODE = 0; 
static unsigned int gBoost_mode_default_freq = 800; // set GPU to 800 MHz in boost mode
static unsigned int gNormal_mode_default_freq = 700; // set GPU to 700 MHz in noarmal mode
static unsigned int gLastFreq_By_Thermal = 0;
static int gpu_freq=700;
static int gpu_benchflix_modify = 0;
static int gpu_youtube=0;
static int gpu_netflix=0;

static unsigned int gDFS_TRIGGER_DEGREE = 90;
typedef struct {
	unsigned int value;
	unsigned int freq;
} FREQ_INDEX;

typedef struct {
	unsigned int value;
	unsigned int freq;
	int thermal;
} FREQ_THERMAL_INDEX;

static FREQ_INDEX  gFREQ_ALL_INDEX[] = {
       {0x02123803, 800}, //f=530 n=56 //0x02123803 <-- 800
       {0x06D13003, 700}, //f=1744 n=48
       {0x012F2D03, 650},
       {0x038e2903, 600},              //0x038e2903 <-- 600
       {0x05ed2503, 550},
       {0X004B2203, 500},   //
       {0x02AA1E03, 450},
       {0x05091A03, 400},
       {0x05091403, 318},
};

static const FREQ_THERMAL_INDEX  gFREQ_INDEX[] = {
	#if 0
	{0x038e2903, 600, -200},
	{0X004B2203, 500, 75},
	{0x02AA1E03, 450, 81},
	{0x05091A03, 400, 87},
	#else
	{0x038e2903, 600, -200},
	{0X004B2203, 500, 95},
	{0x02AA1E03, 450, 105},
	{0x05091A03, 400, 115},
	#endif
};
extern void init_rtk_thermal_sensor(void);
extern int rtk_get_thermal_value(void);
extern int register_temperature_callback(int degree,void *fn, void* data, char *module_name);
extern int register_gpu_dvfs_callback(void *fn);
static u32 get_gpu_freq(void);
extern int gpu_usage;

u32 SysReadHWReg(void* pvLinRegBaseAddr, u32 ui32Offset)
{
	return rtd_inl((long unsigned int)((u8 *)pvLinRegBaseAddr+ui32Offset));
}

void SysWriteHWReg(void *pvLinRegBaseAddr, u32 ui32Offset, u32 ui32Value)
{
    rtd_outl((long unsigned int)((u8 *)pvLinRegBaseAddr+ui32Offset),ui32Value);
}

void set_gpu_dvfs_callback(int dfsmode, unsigned int freq);


//static unsigned int gLastThermal_CutPoint = 0;

//static unsigned int gLastFreq_By_DDK = 0;

// -10 Mhz <  > 10 Mhz
static int isSameFreq(int in1, int in2 )
{
	if( abs(in1 - in2) <= 10) 
		return 0;//same freq
	return -1; 
}

int set_target_freq(unsigned int reg_value, unsigned int target_freq)
{
	uint32_t reg;
	int count = 0;

	mutex_lock(&gGPU_DFS_LOCK);
	//call CRT lock  [FIXME]? => indepdendent reg, don't need ?

	// set PLLGPU_OC_EN=0
	reg = SysReadHWReg((void *)SYS_REG_BASE, SYS_REG_GPU_DVFS_OFFSET) & ~(SYS_REG_GPU_DVFS_OE_BIT);
	SysWriteHWReg((void *)SYS_REG_BASE, SYS_REG_GPU_DVFS_OFFSET, reg);    
	//udelay(0x10);

	//set clock
	SysWriteHWReg((void *)SYS_REG_BASE, SYS_REG_GPU_PLL_GPU2_OFFSET, reg_value);
	//udelay(0x10);

	// set PLLGPU_OC_EN=1
	reg = SysReadHWReg((void *)SYS_REG_BASE, SYS_REG_GPU_DVFS_OFFSET) | (SYS_REG_GPU_DVFS_OE_BIT);
	SysWriteHWReg((void *)SYS_REG_BASE, SYS_REG_GPU_DVFS_OFFSET, reg);
	//MALI_DEBUG_PRINT(2,("ddk SYS_REG_GPU_DVFS_OFFSET[%x] 0x%x\n", 0xb8000448, SysReadHWReg((void *)SYS_REG_BASE, SYS_REG_GPU_DVFS_OFFSET))) ;    
	//printk("ddk [%x] 0x%x\n", 0xb8000448, OSReadHWReg32((IMG_PVOID)SYS_REG_BASE, SYS_REG_GPU_DVFS_OFFSET)) ;

	// delay
	//WAIT_C(0xB8000448,0x2,0x2);		  // wait PLLGPU OC_DONE=1

	while((SysReadHWReg((void *)SYS_REG_BASE, SYS_REG_GPU_DVFS_OFFSET) & SYS_REG_GPU_DVFS_OE_DONE_BIT) != SYS_REG_GPU_DVFS_OE_DONE_BIT)
	{
		if(count++ > 2) {
			//MALI_DEBUG_PRINT_ERROR(("gpu wait freq change fail !!! 0x%x\n", (SysReadHWReg((void *)SYS_REG_BASE, SYS_REG_GPU_DVFS_OFFSET)) ));
			//dump_stack();
			break;
		}
        udelay(10);
	}
	/*
	while((OSReadHWReg32((void *)SYS_REG_BASE, SYS_REG_GPU_DVFS_OFFSET) & _BIT1) != _BIT1)
	{
        	udelay(10);
                if(count++ > 100) 
		{
                         printk(KERN_ERR "gpu wait freq change fail !!! 0x%x\n", (OSReadHWReg32((void *)SYS_REG_BASE, SYS_REG_GPU_DVFS_OFFSET)) );
                         dump_stack();
                         break;
                 }
         }
	*/
	//ex: SysWriteHWReg((void *)REG_GPU_BASE, REG_GPU_PWR_CONFIG_OFF, reg);

	//gLastFreq_By_DDK = get_gpu_freq();
	
 	mutex_unlock(&gGPU_DFS_LOCK);

	//MALI_DEBUG_PRINT(2,("set gpu to %u regValue:0x%x \n", get_gpu_freq(), reg_value));

	return 0;
}


int ddk_set_target_freq(unsigned int reg_value, unsigned int target_freq, unsigned int target_real_freq)
{
	int err = 0;
	int clk_result = get_gpu_freq();
	if ( isSameFreq(clk_result, target_freq) == 0)
	{
		//MALI_DEBUG_PRINT(2,("Same frequency: set gpu CLK %u -> %u regValue:0x%x, no change \n", clk_result, target_freq, reg_value));
		return err;
	}
    if(gpu_benchflix_modify)
		return 0;
	
    if(gkbdev != NULL)
		kbase_pm_suspend(gkbdev);
	set_target_freq( reg_value, target_freq );			
    if(gkbdev != NULL)
		kbase_pm_resume(gkbdev);
	//MALI_DEBUG_PRINT(2,("set gpu CLK %u -> %u \n", clk_result, target_freq));
	return err;
}

static ssize_t gpu_dfs_active_show(struct kobject *kobj, struct kobj_attribute *attr, char *buf)
{
	
	if( gGPU_DFS_MODE == 0 ) {
		return sprintf(buf, "%u:no auto mode\n", gGPU_DFS_MODE);
	}
	else if( gGPU_DFS_MODE == 1 ) {
			return sprintf(buf, "%u:auto(may overwrite by others)\n", gGPU_DFS_MODE);
		}

	return 0;
}

static ssize_t gpu_dfs_active_store(struct kobject *kobj, struct kobj_attribute *attr, const char *buf, size_t count)
{
	int ret = 0;
	if (count <= 0 )
		return -EINVAL;
	
	switch (buf[0]) {
	case '0':
		gGPU_DFS_MODE = 0;
		gLastFreq_By_Thermal = 0;
		ret = 0;
		printk(KERN_WARNING "[Mali] gpu auto DFS will be disabled!!!\n");
		
		break;
	case '1':
		gGPU_DFS_MODE = 1;
		gLastFreq_By_Thermal = 0;
		ret = 0;
		printk(KERN_WARNING "[Mali] gpu auto DFS is enabled!\n");
		break;
	default:
		ret = -EINVAL;
	}

	return (ret >= 0) ? count : ret;
}


static ssize_t gpu_dfs_outstand_show(struct kobject *kobj, struct kobj_attribute *attr, char *buf)
{
	//unsigned int r,w;
	
        //r = ((SysReadHWReg((void *)REG_GPU_BASE, 0xE030)) >> 16) & 0x3F;
	//w = ((SysReadHWReg((void *)REG_GPU_BASE, 0xE030)) ) & 0x3F;
	//return sprintf(buf, "%u %u T:%d\n", r, w,  rtk_get_thermal_value() );
		
	return sprintf(buf, "Current thermal value : %d\n", rtk_get_thermal_value() );
}

static ssize_t gpu_dfs_outstand_store(struct kobject *kobj, struct kobj_attribute *attr, const char *buf, size_t count)
{
	return -1;
}


static ssize_t gpu_dfs_freq400_show(struct kobject *kobj, struct kobj_attribute *attr, char *buf)
{
	if(gpu_freq != 400)
		ddk_set_target_freq(0x05091403, 400, 400); //0x05091403 --> 318M
	gpu_freq = 400;
	gpu_benchflix_modify = 1;
	return 0;
}

static ssize_t gpu_dfs_freq400_store(struct kobject *kobj, struct kobj_attribute *attr, const char *buf, size_t count)
{
	return -1;
}
extern int otp_field_get_production_grade(unsigned int *);
static ssize_t gpu_dfs_freq800_show(struct kobject *kobj, struct kobj_attribute *attr, char *buf)
{
	if(gpu_freq != 800)
		ddk_set_target_freq(0x02123803, 800, 800);

	gpu_freq = 800;
	gpu_benchflix_modify = 1;
	return 0;
}

static ssize_t gpu_dfs_youtube_in_show(struct kobject *kobj, struct kobj_attribute *attr, char *buf)
{
       // printk("func=%s line=%d \n ",__FUNCTION__,__LINE__);
	gpu_youtube = 1;

	return 0;
}

static ssize_t gpu_dfs_youtube_out_show(struct kobject *kobj, struct kobj_attribute *attr, char *buf)
{
       // printk("func=%s line=%d \n ",__FUNCTION__,__LINE__);
	gpu_youtube = 0;

	return 0;
}

static ssize_t gpu_dfs_youtube_read_show(struct kobject *kobj, struct kobj_attribute *attr, char *buf)
{
         //printk("func=%s line=%d gpu_youtube=%d\n ",__FUNCTION__,__LINE__,gpu_youtube);
	return sprintf(buf, "%u\n", gpu_youtube );
}

static ssize_t gpu_dfs_netflix_in_show(struct kobject *kobj, struct kobj_attribute *attr, char *buf)
{
	gpu_netflix = 1;

	return 0;
}

static ssize_t gpu_dfs_netflix_out_show(struct kobject *kobj, struct kobj_attribute *attr, char *buf)
{
	gpu_netflix = 0;

	return 0;
}

static ssize_t gpu_dfs_netflix_read_show(struct kobject *kobj, struct kobj_attribute *attr, char *buf)
{
	return sprintf(buf, "%u\n", gpu_netflix );
}
#if 0
static ssize_t gpu_dfs_step11_read_show(struct kobject *kobj, struct kobj_attribute *attr, char *buf)
{
	//memcdtg_DV_video_blending_region
    rtd_outl(0xb8028620,0x0132034c);
    //memcdtg_dh_video_blending_region
    rtd_outl(0xb8028624,0x013204f2);
	//use video_den
	rtd_outl(0xb802b000,rtd_inl(0xb802b000) & ~(0x01000000));
	
	//Mixer_b3
	rtd_outl(0xb802b06c,0x003f003f);

	//apply mixer 
	rtd_outl(0xb802B0A0,0x3);
	
	return sprintf(buf, "%u\n", 11 );
}

static ssize_t gpu_dfs_step12_read_show(struct kobject *kobj, struct kobj_attribute *attr, char *buf)
{
	//memcdtg_DV_video_blending_region
    rtd_outl(0xb8028620,0x01320316);
    //memcdtg_dh_video_blending_region
    rtd_outl(0xb8028624,0x01320492);
	//use video_den
	rtd_outl(0xb802b000,rtd_inl(0xb802b000) & ~(0x01000000));
	
	//Mixer_b3
	rtd_outl(0xb802b06c,0x003f003f);
	//Mixer apply
	rtd_outl(0xb802B0A0,0x3);

	return sprintf(buf, "%u\n", 12 );
}

static ssize_t gpu_dfs_step00_read_show(struct kobject *kobj, struct kobj_attribute *attr, char *buf)
{
	//memcdtg_DV_video_blending_region
    rtd_outl(0xb8028620,0x0);
    //memcdtg_dh_video_blending_region
    rtd_outl(0xb8028624,0x0);
	//use video_den
	rtd_outl(0xb802b000,rtd_inl(0xb802b000) | (0x01000000));
	//Mixer_b3
	rtd_outl(0xb802b06c,0x01040104);
	//Mixer apply
	rtd_outl(0xb802B0A0,0x3);
	return sprintf(buf, "%u\n", 00 );
}

static ssize_t gpu_dfs_step14_read_show(struct kobject *kobj, struct kobj_attribute *attr, char *buf)
{
	//memcdtg_DV_video_blending_region
    rtd_outl(0xb8028620,0x01320330);
    //memcdtg_dh_video_blending_region
    rtd_outl(0xb8028624,0x013204c2);
	//use video_den
	rtd_outl(0xb802b000,rtd_inl(0xb802b000) & ~(0x01000000));
	
	//Mixer_b3
	rtd_outl(0xb802b06c,0x003f003f);
	//Mixer apply
	rtd_outl(0xb802B0A0,0x3);

	return sprintf(buf, "%u\n", 14 );
}
static ssize_t gpu_dfs_step15_read_show(struct kobject *kobj, struct kobj_attribute *attr, char *buf)
{
	//memcdtg_DV_video_blending_region
    rtd_outl(0xb8028620,0x01360334);
    //memcdtg_dh_video_blending_region
    rtd_outl(0xb8028624,0x013604c4);
	//use video_den
	rtd_outl(0xb802b000,rtd_inl(0xb802b000) & ~(0x01000000));
	
	//Mixer_b3
	rtd_outl(0xb802b06c,0x003f003f);
	//Mixer apply
	rtd_outl(0xb802B0A0,0x3);

	return sprintf(buf, "%u\n", 15 );
}
static ssize_t gpu_dfs_step16_read_show(struct kobject *kobj, struct kobj_attribute *attr, char *buf)
{
	//memcdtg_DV_video_blending_region
    rtd_outl(0xb8028620,0x012c02dc);
    //memcdtg_dh_video_blending_region
    rtd_outl(0xb8028624,0x016E046e);
	//use video_den
	rtd_outl(0xb802b000,rtd_inl(0xb802b000) & ~(0x01000000));
	
	//Mixer_b3
	rtd_outl(0xb802b06c,0x003f003f);
	//Mixer apply
	rtd_outl(0xb802B0A0,0x3);

	return sprintf(buf, "%u\n", 16 );
}
static ssize_t gpu_dfs_step17_read_show(struct kobject *kobj, struct kobj_attribute *attr, char *buf)
{
	//memcdtg_DV_video_blending_region
    rtd_outl(0xb8028620,0x01660360);
    //memcdtg_dh_video_blending_region
    rtd_outl(0xb8028624,0x012c04B0);
	//use video_den
	rtd_outl(0xb802b000,rtd_inl(0xb802b000) & ~(0x01000000));
	
	//Mixer_b3
	rtd_outl(0xb802b06c,0x003f003f);
	//Mixer apply
	rtd_outl(0xb802B0A0,0x3);

	return sprintf(buf, "%u\n", 17 );
}
static ssize_t gpu_dfs_step109_read_show(struct kobject *kobj, struct kobj_attribute *attr, char *buf)
{
	//memcdtg_DV_video_blending_region
    rtd_outl(0xb8028620,0x012c02dc);
    //memcdtg_dh_video_blending_region
    rtd_outl(0xb8028624,0x012c042c);
	//use video_den
	rtd_outl(0xb802b000,rtd_inl(0xb802b000) & ~(0x01000000));
	
	//Mixer_b3
	rtd_outl(0xb802b06c,0x003f003f);
	//Mixer apply
	rtd_outl(0xb802B0A0,0x3);

	return sprintf(buf, "%u\n", 17 );
}
#endif

static ssize_t gpu_dfs_freq800_store(struct kobject *kobj, struct kobj_attribute *attr, const char *buf, size_t count)
{
	return -1;
}
static ssize_t gpu_dfs_freq700_show(struct kobject *kobj, struct kobj_attribute *attr, char *buf)
{
	gpu_benchflix_modify = 0;
	
	if(gpu_freq != 700)
		ddk_set_target_freq(0x06D13003, 700, 700);
	gpu_freq = 700;

	return -1;		
}

static ssize_t gpu_dfs_freq700_store(struct kobject *kobj, struct kobj_attribute *attr, const char *buf, size_t count)
{
	return -1;
}


///MHZ
static u32 get_gpu_freq(void)
{
	unsigned int freq, tmp;
	unsigned int gpu_pre_div;
	unsigned int value_f,value_n ;

	tmp = (SysReadHWReg((void *)SYS_REG_BASE,SYS_REG_GPU_PLL_GPU2_OFFSET));
	gpu_pre_div = ((rtd_inl((unsigned int)0xB8000450) & 0x30) >> 4)+1;
	value_f = ((tmp & 0x07FF0000) >> 16);
	value_n = ((tmp & 0x0000FF00) >> 8);
	freq = 27 * (value_n + 3) + ((27 * value_f) >> 11); // PLL Frequency = 27*(NCODE +3+ FCODE/2048)
	freq = freq/gpu_pre_div;

	return freq;
}

static ssize_t gpu_dfs_usage_show(struct kobject *kobj, struct kobj_attribute *attr, char *buf)
{
	return sprintf(buf, "%u\n",gpu_usage );
}


static ssize_t gpu_dfs_freq_show(struct kobject *kobj, struct kobj_attribute *attr, char *buf)
{
	return sprintf(buf, "%u\n",get_gpu_freq() );
}

static ssize_t gpu_dfs_freq_store(struct kobject *kobj, struct kobj_attribute *attr, const char *buf, size_t count)
{
#if 1
	int ret;
	int i=0;
	unsigned int in_val = 0;
	int num_idx = sizeof(gFREQ_ALL_INDEX)/sizeof(gFREQ_ALL_INDEX[0]);
	unsigned int target_freq = 0;
	unsigned int target_reg_val = -1;

	
	ret = sscanf(buf, "%u", &in_val);
	if (ret != 1) {
		printk(KERN_ERR " gpu dfs get freq val fail %s\n", buf);
		return -EINVAL;
	}

	//not below 400Mhz currently
	if( in_val < 400) {

		printk(KERN_ERR " gpu dfs need >= 400. cur input is %u\n", in_val);
		return -EINVAL; 
	}

	for(i = 0 ; i < num_idx ; i++ ) 
	{
		if( in_val >= gFREQ_ALL_INDEX[i].freq ) {
			target_freq = gFREQ_ALL_INDEX[i].freq;
			target_reg_val = gFREQ_ALL_INDEX[i].value;
			break;
		}
	}

	if( target_reg_val != -1) {
		ddk_set_target_freq(target_reg_val, target_freq, in_val);
	}

	return count;
#else

	return 0;

#endif

}

static struct kobj_attribute gpu_dfs_active_attr = 
	__ATTR(dvfsMode, 0644, gpu_dfs_active_show, gpu_dfs_active_store);

static struct kobj_attribute gpu_dfs_usage_attr = 
	__ATTR(usage, 0644, gpu_dfs_usage_show, gpu_dfs_freq_store);

static struct kobj_attribute gpu_dfs_freq_attr = 
	__ATTR(freq, 0644, gpu_dfs_freq_show, gpu_dfs_freq_store);

static struct kobj_attribute gpu_dfs_outstanding_attr = 
	__ATTR(outstand, 0644, gpu_dfs_outstand_show, gpu_dfs_outstand_store);

static struct kobj_attribute gpu_dfs_freq400_attr = 
	__ATTR(freq400, 0644, gpu_dfs_freq400_show, gpu_dfs_freq400_store);

static struct kobj_attribute gpu_dfs_freq800_attr = 
	__ATTR(freq800, 0644, gpu_dfs_freq800_show, gpu_dfs_freq800_store);

static struct kobj_attribute gpu_dfs_freq700_attr = 
	__ATTR(freq700, 0644, gpu_dfs_freq700_show, gpu_dfs_freq700_store);


static struct kobj_attribute gpu_dfs_youtube_in_attr = 
	__ATTR(youtube_in, 0644, gpu_dfs_youtube_in_show, gpu_dfs_freq_store);
	
static struct kobj_attribute gpu_dfs_youtube_out_attr = 
	__ATTR(youtube_out, 0644, gpu_dfs_youtube_out_show, gpu_dfs_freq_store);

static struct kobj_attribute gpu_dfs_youtube_read_attr = 
	__ATTR(youtube_read, 0644, gpu_dfs_youtube_read_show, gpu_dfs_freq_store);
	
static struct kobj_attribute gpu_dfs_netflix_in_attr = 
	__ATTR(netflix_in, 0644, gpu_dfs_netflix_in_show, gpu_dfs_freq_store);

static struct kobj_attribute gpu_dfs_netflix_out_attr = 
	__ATTR(netflix_out, 0644, gpu_dfs_netflix_out_show, gpu_dfs_freq_store);

static struct kobj_attribute gpu_dfs_netflix_read_attr = 
	__ATTR(netflix_read, 0644, gpu_dfs_netflix_read_show, gpu_dfs_freq_store);

#if 0
static struct kobj_attribute gpu_dfs_netflix_step11_read_attr = 
	__ATTR(step11_read, 0644, gpu_dfs_step11_read_show, gpu_dfs_freq_store);

static struct kobj_attribute gpu_dfs_netflix_step12_read_attr = 
	__ATTR(step12_read, 0644, gpu_dfs_step12_read_show, gpu_dfs_freq_store);

static struct kobj_attribute gpu_dfs_netflix_step00_read_attr = 
	__ATTR(step00_read, 0644, gpu_dfs_step00_read_show, gpu_dfs_freq_store);

static struct kobj_attribute gpu_dfs_netflix_step14_read_attr = 
	__ATTR(step14_read, 0644, gpu_dfs_step14_read_show, gpu_dfs_freq_store);

static struct kobj_attribute gpu_dfs_netflix_step15_read_attr = 
	__ATTR(step15_read, 0644, gpu_dfs_step15_read_show, gpu_dfs_freq_store);

static struct kobj_attribute gpu_dfs_netflix_step16_read_attr = 
	__ATTR(step16_read, 0644, gpu_dfs_step16_read_show, gpu_dfs_freq_store);
			
static struct kobj_attribute gpu_dfs_netflix_step17_read_attr = 
	__ATTR(step17_read, 0644, gpu_dfs_step17_read_show, gpu_dfs_freq_store);

static struct kobj_attribute gpu_dfs_netflix_step109_read_attr = 
	__ATTR(step109_read, 0644, gpu_dfs_step109_read_show, gpu_dfs_freq_store);
#endif

			
static struct kobject *gGPU_DFS_KOBJ;
extern struct kobject *realtek_boards_kobj;
	
int rt_gpu_dfs_sysfs_init(void)
{
	int ret;

//	rtk_monitor_kobj = kobject_create_and_add("rtk_monitor", kernel_kobj);
//	if (!rtk_monitor_kobj)
//		return -ENOMEM;
//	ret = sysfs_create_group(rtk_monitor_kobj, &rtk_monitor_attr_group);
//	if (ret)
//		kobject_put(rtk_monitor_kobj);

	//"ASSUME" [1].thermal is lowest tempertaure
	gDFS_TRIGGER_DEGREE = gFREQ_INDEX[1].thermal;

	if (!realtek_boards_kobj)
		return -ENOMEM;

#if 0
	gGPU_DFS_KOBJ = kobject_create();
	ret = kobject_set_name(gGPU_DFS_KOBJ, "gpu");
	if (ret)
		goto out;
	ret = kobject_add(gGPU_DFS_KOBJ, realtek_boards_kobj, "gpu");
	if (ret)
		goto out;
#else
	gGPU_DFS_KOBJ = kobject_create_and_add("gpu", realtek_boards_kobj); 
#endif

	ret = sysfs_create_file(gGPU_DFS_KOBJ, &gpu_dfs_active_attr.attr);
	ret = sysfs_create_file(gGPU_DFS_KOBJ, &gpu_dfs_usage_attr.attr);
	ret |= sysfs_create_file(gGPU_DFS_KOBJ, &gpu_dfs_freq_attr.attr);
	ret |= sysfs_create_file(gGPU_DFS_KOBJ, &gpu_dfs_outstanding_attr.attr);
	ret |= sysfs_create_file(gGPU_DFS_KOBJ, &gpu_dfs_freq400_attr.attr);
	ret |= sysfs_create_file(gGPU_DFS_KOBJ, &gpu_dfs_freq800_attr.attr);
	ret |= sysfs_create_file(gGPU_DFS_KOBJ, &gpu_dfs_freq700_attr.attr);
	
	ret |= sysfs_create_file(gGPU_DFS_KOBJ, &gpu_dfs_youtube_in_attr.attr);
	ret |= sysfs_create_file(gGPU_DFS_KOBJ, &gpu_dfs_youtube_out_attr.attr);	
	ret |= sysfs_create_file(gGPU_DFS_KOBJ, &gpu_dfs_youtube_read_attr.attr);

	ret |= sysfs_create_file(gGPU_DFS_KOBJ, &gpu_dfs_netflix_in_attr.attr);
	ret |= sysfs_create_file(gGPU_DFS_KOBJ, &gpu_dfs_netflix_out_attr.attr);	
	ret |= sysfs_create_file(gGPU_DFS_KOBJ, &gpu_dfs_netflix_read_attr.attr);
#if 0	
	ret |= sysfs_create_file(gGPU_DFS_KOBJ, &gpu_dfs_netflix_step11_read_attr.attr);
	ret |= sysfs_create_file(gGPU_DFS_KOBJ, &gpu_dfs_netflix_step12_read_attr.attr);
	ret |= sysfs_create_file(gGPU_DFS_KOBJ, &gpu_dfs_netflix_step00_read_attr.attr);
	
	ret |= sysfs_create_file(gGPU_DFS_KOBJ, &gpu_dfs_netflix_step14_read_attr.attr);
	ret |= sysfs_create_file(gGPU_DFS_KOBJ, &gpu_dfs_netflix_step15_read_attr.attr);
	ret |= sysfs_create_file(gGPU_DFS_KOBJ, &gpu_dfs_netflix_step16_read_attr.attr);
	ret |= sysfs_create_file(gGPU_DFS_KOBJ, &gpu_dfs_netflix_step17_read_attr.attr);
	ret |= sysfs_create_file(gGPU_DFS_KOBJ, &gpu_dfs_netflix_step109_read_attr.attr);	
#endif
	if (ret)
		kobject_put(gGPU_DFS_KOBJ);		
	return ret;
}


void GPU_thermal_callback(void *data, int cur_degree , int reg_degree, char *module_name)
{
	int thermal = 0;
	unsigned int in_freq = 0;
	int thermal_levels = 0;	
	int i =0 ;

	thermal = cur_degree;
	//MALI_DEBUG_PRINT(2,("GPU_thermal_callback is called: current thermal=%d, reg_degree=%d\n", cur_degree, reg_degree));
	/* note: if through sys, don't set here anymore */
	if( gGPU_DFS_MODE == 0) 
	{
		//MALI_DEBUG_PRINT(2,("auto dfs mode is disabled\n"));
		return ;
	}

	thermal_levels = sizeof(gFREQ_INDEX)/sizeof(gFREQ_INDEX[0]);
	in_freq = gFREQ_INDEX[thermal_levels - 1].freq;

  	//generic code logic
	for(i= thermal_levels-1; i >= 0; i--) 
	{		
		if( thermal >= gFREQ_INDEX[i].thermal) 
		{			
			in_freq = gFREQ_INDEX[i].freq;			
			break;
		}
	}

	if( gLastFreq_By_Thermal == in_freq || ( isSameFreq( gLastFreq_By_Thermal, in_freq) == 0))
	{
		//don't need set
		return ;
	}
	
	gLastFreq_By_Thermal = in_freq;

	{
		int num_idx = sizeof(gFREQ_INDEX)/sizeof(gFREQ_INDEX[0]);
		unsigned int target_freq = 0;
		unsigned int target_reg_val = -1;
		

		for(i = 0 ; i < num_idx ; i++) 
		{
			if( in_freq >= gFREQ_INDEX[i].freq ) 
			{
				target_freq = gFREQ_INDEX[i].freq;
				target_reg_val = gFREQ_INDEX[i].value;
				break;
			}
				
		}
		
		if( target_reg_val != -1) 
		{
			ddk_set_target_freq(target_reg_val, target_freq, in_freq);		
		}

	}	

    return;
}

/* set_gpu_dvfs_callback(int dfsmode, unsigned int freq)
*  dfsmode=0 disable gpu DFS by thermal
*  dfsmode=1 enable gpu DFS by thermal
*freq:valid value 0 800 700 650 600 550 500 450 400
*if freq=0, set to default freq: boost 800, normal 600
*/
void set_gpu_dvfs_callback(int dfsmode, unsigned int freq)
{
	int num_idx = sizeof(gFREQ_ALL_INDEX)/sizeof(gFREQ_ALL_INDEX[0]);
	int i = 0;
	unsigned int target_freq = 0;
	unsigned int target_reg_val = -1;
	
	if( !dfsmode )
	{
		if (freq == 0)
		{
			freq = gBoost_mode_default_freq;
			//MALI_DEBUG_PRINT(2,("set_gpu_dvfs_callback: set gpu to default freq %uMhz\n",gBoost_mode_default_freq));
		}
		gGPU_DFS_MODE = 0;
		gLastFreq_By_Thermal = 0;
		for(i = 0 ; i < num_idx ; i++) 
		{
			if( freq >= gFREQ_ALL_INDEX[i].freq ) 
			{
				target_freq = gFREQ_ALL_INDEX[i].freq;
				target_reg_val = gFREQ_ALL_INDEX[i].value;
				break;
			}
				
		}
		
		if( target_reg_val != -1) 
		{
			ddk_set_target_freq(target_reg_val, target_freq, freq);		
		}


	}else{
		if (freq == 0)
		{
			freq = gNormal_mode_default_freq;
			//MALI_DEBUG_PRINT(2,("set_gpu_dvfs_callback:set gpu normal mode to default freq %uMhz\n",gNormal_mode_default_freq));
		}

		gGPU_DFS_MODE = 1;
		gLastFreq_By_Thermal = 0;
		for(i = 0 ; i < num_idx ; i++) 
		{
			if( freq >= gFREQ_ALL_INDEX[i].freq ) 
			{
				target_freq = gFREQ_ALL_INDEX[i].freq;
				target_reg_val = gFREQ_ALL_INDEX[i].value;
				break;
			}
		}
		
		if( target_reg_val != -1) 
		{
			ddk_set_target_freq(target_reg_val, target_freq, freq);		
		}

	}

}


int rt_gpu_dfs_init(struct kbase_device *kbdev)
{	
	int err = 0;
	int ret=0;
	u32 reg;
	
	gkbdev = kbdev;
	//set PLLGPU_PDIV to 1: divide by 2
	reg = SysReadHWReg((void *)SYS_REG_BASE, SYS_REG_GPU_PLL_GPU1_OFFSET);
	SysWriteHWReg((void *)SYS_REG_BASE, SYS_REG_GPU_PLL_GPU1_OFFSET, (reg & (~0x30)) | 0x10);

	//MALI_DEBUG_PRINT(2,(" In rt_gpu_dfs_init, Reg : PLLGPU_PDIV value = 0x%x\n",  SysReadHWReg((void *)SYS_REG_BASE, SYS_REG_GPU_PLL_GPU1_OFFSET)));
	init_rtk_thermal_sensor();	
	ret = rt_gpu_dfs_sysfs_init();
	if (ret )
	{
		//MALI_PRINT_ERROR(("rt_gpu_dfs_sysfs_init() failed !\n"));
		//err = _MALI_OSK_ERR_FAULT;
		return err;
	}

	register_gpu_dvfs_callback(set_gpu_dvfs_callback );

	if ( (ret = register_temperature_callback( gDFS_TRIGGER_DEGREE, GPU_thermal_callback, &gGPU_DFS_MODE, "gpu")) != 0)
	{
		//MALI_PRINT_ERROR(("register gpu thermal handler fail, ret:%d \n", ret));	
		//err = _MALI_OSK_ERR_FAULT;
	}
	
	return err;
}


